@@ -1,7 +1,7 @@
 extern crate libc;
 
 use array::Array;
-use defines::{AfError, BorderType, ColorSpace, Connectivity, InterpType, YCCStd};
+use defines::{AfError, BorderType, ColorSpace, Connectivity, InterpType, YCCStd, MomentType};
 use error::HANDLE_ERROR;
 use util::HasAfEnum;
 use self::libc::{uint8_t, c_uint, c_int, c_float, c_double};
@@ -65,6 +65,8 @@ extern {
     fn af_medfilt(out: MutAfArray, input: AfArray,
                   wlen: DimT, wwid: DimT, etype: uint8_t) -> c_int;
 
+    fn af_medfilt1(out: MutAfArray, input: AfArray, wlen: DimT, etype: uint8_t) -> c_int;
+
     fn af_minfilt(out: MutAfArray, input: AfArray,
                   wlen: DimT, wwid: DimT, etype: uint8_t) -> c_int;
 
@@ -93,6 +95,9 @@ extern {
     fn af_rgb2ycbcr(out: MutAfArray, input: AfArray, stnd: c_int) -> c_int;
     fn af_is_image_io_available(out: *mut c_int) -> c_int;
     fn af_transform_coordinates(out: MutAfArray, tf: AfArray, d0: c_float, d1: c_float) -> c_int;
+
+    fn af_moments(out: MutAfArray, input: AfArray, moment: c_int) ->c_int;
+    fn af_moments_all(out: *mut c_double, input: AfArray, moment: c_int) ->c_int;
 }
 
 /// Calculate the gradients
@@ -652,7 +657,19 @@ pub fn mean_shift(input: &Array, spatial_sigma: f32, chromatic_sigma: f32,
 }
 
 macro_rules! filt_func_def {
-    ($fn_name: ident, $ffi_name: ident) => (
+    ($doc_str: expr, $fn_name: ident, $ffi_name: ident) => (
+        #[doc=$doc_str]
+        ///
+        ///# Parameters
+        ///
+        /// - `input` is the input image(Array)
+        /// - `wlen` is the horizontal length of the filter
+        /// - `hlen` is the vertical length of the filter
+        /// - `etype` is enum of type [BorderType](./enum.BorderType.html)
+        ///
+        ///# Return Values
+        ///
+        /// An Array with filtered image data.
         #[allow(unused_mut)]
         pub fn $fn_name(input: &Array, wlen: u64, wwid: u64,
                         etype: BorderType) -> Array {
@@ -667,9 +684,9 @@ macro_rules! filt_func_def {
     )
 }
 
-filt_func_def!(medfilt, af_medfilt);
-filt_func_def!(minfilt, af_minfilt);
-filt_func_def!(maxfilt, af_maxfilt);
+filt_func_def!("Median filter", medfilt, af_medfilt);
+filt_func_def!("Box filter with minimum as box operation", minfilt, af_minfilt);
+filt_func_def!("Box filter with maximum as box operation", maxfilt, af_maxfilt);
 
 /// Creates a Gaussian Kernel.
 ///
@@ -830,14 +847,18 @@ pub fn hist_equal(input: &Array, hist: &Array) -> Array {
 }
 
 macro_rules! grayrgb_func_def {
-    ($fn_name: ident, $ffi_name: ident) => (
-        /// Color space conversion functions
+    ($doc_str: expr, $fn_name: ident, $ffi_name: ident) => (
+        #[doc=$doc_str]
         ///
-        /// # Parameters
+        ///# Parameters
         ///
         /// - `r` is fraction of red channel to appear in output
         /// - `g` is fraction of green channel to appear in output
         /// - `b` is fraction of blue channel to appear in output
+        ///
+        ///#Return Values
+        ///
+        ///An Array with image data in target color space
         #[allow(unused_mut)]
         pub fn $fn_name(input: &Array, r: f32, g: f32, b: f32) -> Array {
             unsafe {
@@ -851,12 +872,12 @@ macro_rules! grayrgb_func_def {
     )
 }
 
-grayrgb_func_def!(rgb2gray, af_rgb2gray);
-grayrgb_func_def!(gray2rgb, af_gray2rgb);
+grayrgb_func_def!("Color(RGB) to Grayscale conversion", rgb2gray, af_rgb2gray);
+grayrgb_func_def!("Grayscale to Color(RGB) conversion", gray2rgb, af_gray2rgb);
 
 macro_rules! hsvrgb_func_def {
-    ($fn_name: ident, $ffi_name: ident) => (
-        /// Color space conversion functions
+    ($doc_str: expr, $fn_name: ident, $ffi_name: ident) => (
+        #[doc=$doc_str]
         #[allow(unused_mut)]
         pub fn $fn_name(input: &Array) -> Array {
             unsafe {
@@ -869,8 +890,8 @@ macro_rules! hsvrgb_func_def {
     )
 }
 
-hsvrgb_func_def!(hsv2rgb, af_hsv2rgb);
-hsvrgb_func_def!(rgb2hsv, af_rgb2hsv);
+hsvrgb_func_def!("HSV to RGB color space conversion", hsv2rgb, af_hsv2rgb);
+hsvrgb_func_def!("RGB to HSV color space conversion", rgb2hsv, af_rgb2hsv);
 
 /// Generate an array with image windows as columns
 ///
@@ -910,7 +931,7 @@ hsvrgb_func_def!(rgb2hsv, af_rgb2hsv);
 ///
 /// # Examples
 ///
-/// ```ignore
+/// ```text
 /// A [5 5 1 1]
 /// 10 15 20 25 30
 /// 11 16 21 26 31
@@ -1119,4 +1140,69 @@ pub fn transform_coords(tf: &Array, d0: f32, d1: f32) -> Array {
         HANDLE_ERROR(AfError::from(err_val));
         Array::from(temp)
     }
-}
\ No newline at end of file
+}
+
+/// Find Image moments
+///
+/// # Parameters
+///
+/// - `input` is the input image
+/// - `moment` is the type of moment to be computed, takes a value of
+/// [enum](./enum.MomentType.html)
+///
+/// # Return Values
+///
+/// Moments Array
+pub fn moments(input: &Array, moment: MomentType) -> Array {
+    unsafe {
+        let mut temp: i64 = 0;
+        let err_val = af_moments(&mut temp as MutAfArray,
+                                 input.get() as AfArray,
+                                 moment as c_int);
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
+    }
+}
+
+/// Find Image moment for whole image
+///
+/// # Parameters
+///
+/// - `input` is the input image
+/// - `moment` is the type of moment to be computed, takes a value of
+/// [enum](./enum.MomentType.html)
+///
+/// # Return Values
+///
+/// Moment value of the whole image
+pub fn moments_all(input: &Array, moment: MomentType) -> f64 {
+    unsafe {
+        let mut temp: f64 = 0.0;
+        let err_val = af_moments_all(&mut temp as *mut c_double,
+                                     input.get() as AfArray,
+                                     moment as c_int);
+        HANDLE_ERROR(AfError::from(err_val));
+        temp
+    }
+}
+
+/// One dimensional median filter on image
+///
+/// # Parameters
+///
+///  - `input` is the input image(Array)
+///  - `wlen` is the horizontal length of the filter
+///  - `etype` is enum of type [BorderType](./enum.BorderType.html)
+///
+/// # Return Values
+///
+/// An Array with filtered image data.
+pub fn medfilt1(input: &Array, wlen: u64, etype: BorderType) -> Array {
+    unsafe {
+        let mut temp: i64 = 0;
+        let err_val = af_medfilt1(&mut temp as MutAfArray, input.get() as AfArray,
+                                  wlen as DimT, etype as uint8_t);
+        HANDLE_ERROR(AfError::from(err_val));
+        Array::from(temp)
+    }
+}
