@@ -6,7 +6,7 @@
 extern crate lazy_static;
 
 pub use array::Array;
-pub use array::{print};
+pub use array::{print, print_gen, eval_multiple, is_eval_manual, set_manual_eval};
 mod array;
 
 //pub use algorithm::{sum_nan, product_nan, sum_nan_all, product_nan_all};
@@ -14,12 +14,12 @@ pub use algorithm::{sum, product, min, max, all_true, any_true, count, sum_nan,
 pub use algorithm::{sum_all, product_all, min_all, max_all, sum_nan_all, product_nan_all};
 pub use algorithm::{all_true_all, any_true_all, count_all, imin, imax, imin_all, imax_all};
 pub use algorithm::{accum, locate, diff1, diff2, sort, sort_index, sort_by_key};
-pub use algorithm::{set_unique, set_union, set_intersect};
+pub use algorithm::{set_unique, set_union, set_intersect, scan, scan_by_key};
 mod algorithm;
 
 pub use arith::{add, sub, div, mul, lt, gt, le, ge, eq, neq, and, or, minof, maxof, rem};
 pub use arith::{bitand, bitor, bitxor, shiftl, shiftr};
-pub use arith::{abs, sign, round, trunc, floor, ceil, modulo, sigmoid};
+pub use arith::{abs, sign, round, trunc, floor, ceil, modulo, sigmoid, clamp};
 pub use arith::{sin, cos, tan, asin, acos, atan, sinh, cosh, tanh, asinh, acosh, atanh};
 pub use arith::{atan2, cplx2, arg, cplx, real, imag, conjg, hypot};
 pub use arith::{sqrt, log, log1p, log10, log2, pow2, exp, expm1, erf, erfc, root, pow};
@@ -33,7 +33,6 @@ pub use blas::{matmul, dot, transpose, transpose_inplace};
 mod blas;
 
 pub use data::{constant, range, iota};
-pub use data::{set_seed, get_seed, randu, randn};
 pub use data::{identity, diag_create, diag_extract, lower, upper};
 pub use data::{join, join_many, tile};
 pub use data::{reorder, shift, moddims, flat, flip};
@@ -47,7 +46,8 @@ mod device;
 pub use defines::{DType, AfError, Backend, ColorMap, YCCStd, HomographyType};
 pub use defines::{InterpType, BorderType, MatchType, NormType};
 pub use defines::{Connectivity, ConvMode, ConvDomain, ColorSpace, MatProp};
-pub use defines::{MarkerType};
+pub use defines::{MarkerType, MomentType, SparseFormat, BinaryOp, RandomEngineType};
+pub use defines::{PHILOX, THREEFRY, MERSENNE, DEFAULT_RANDOM_ENGINE};
 mod defines;
 
 pub use dim4::Dim4;
@@ -72,8 +72,9 @@ pub use image::{resize, transform, rotate, translate, scale, skew};
 pub use image::{dilate, dilate3, erode, erode3, minfilt, maxfilt};
 pub use image::{gradient, histogram, hist_equal, regions};
 pub use image::{gray2rgb, rgb2gray, hsv2rgb, rgb2hsv, color_space};
-pub use image::{bilateral, mean_shift, medfilt, sobel};
+pub use image::{bilateral, mean_shift, medfilt, sobel, medfilt1};
 pub use image::{unwrap, wrap, sat, rgb2ycbcr, ycbcr2rgb, is_imageio_available, transform_coords};
+pub use image::{moments, moments_all};
 mod image;
 
 pub use lapack::{svd, lu, qr, cholesky, solve, solve_lu, inverse, det, rank, norm};
@@ -82,7 +83,12 @@ mod lapack;
 mod macros;
 mod num;
 
-pub use signal::{approx1, approx2};
+pub use random::RandomEngine;
+pub use random::{set_seed, get_seed, randu, randn, random_uniform, random_normal};
+pub use random::{get_default_random_engine, set_default_random_engine_type};
+mod random;
+
+pub use signal::{approx1, approx2, set_fft_plan_cache_size};
 pub use signal::{fft, fft2, fft3, ifft, ifft2, ifft3};
 pub use signal::{fft_r2c, fft2_r2c, fft3_r2c, fft_c2r, fft2_c2r, fft3_c2r};
 pub use signal::{fft_inplace, fft2_inplace, fft3_inplace};
@@ -92,13 +98,18 @@ pub use signal::{fft_convolve1, fft_convolve2, fft_convolve3};
 pub use signal::{fir, iir};
 mod signal;
 
+pub use sparse::{sparse, sparse_from_host, sparse_from_dense, sparse_convert_to};
+pub use sparse::{sparse_to_dense, sparse_get_info, sparse_get_values, sparse_get_nnz};
+pub use sparse::{sparse_get_row_indices, sparse_get_col_indices, sparse_get_format};
+mod sparse;
+
 pub use statistics::{mean, stdev, median, var, cov, corrcoef};
 pub use statistics::{mean_weighted, var_weighted};
 pub use statistics::{var_all, mean_all, stdev_all, median_all};
 pub use statistics::{mean_all_weighted, var_all_weighted};
 mod statistics;
 
-pub use util::{HasAfEnum};
+pub use util::{HasAfEnum, get_size};
 mod util;
 
 pub use vision::Features;
