@@ -1,8 +1,27 @@
+extern crate libc;
 extern crate num;
 
 use defines::{AfError, ColorMap, ConvDomain, ConvMode, DType, InterpType, MatProp, MatchType};
+use defines::{SparseFormat, BinaryOp, RandomEngineType};
+use error::HANDLE_ERROR;
 use std::mem;
 use self::num::Complex;
+use self::libc::{uint8_t, c_int, size_t};
+
+#[allow(dead_code)]
+extern {
+    fn af_get_size_of(size: *mut size_t, aftype: uint8_t) -> c_int;
+}
+
+/// Get size, in bytes, of the arrayfire native type
+pub fn get_size(value: DType) -> usize {
+    unsafe {
+        let mut ret_val: usize = 0;
+        let err_val = af_get_size_of(&mut ret_val as *mut size_t, value as uint8_t);
+        HANDLE_ERROR(AfError::from(err_val));
+        ret_val
+    }
+}
 
 impl From<i32> for AfError {
     fn from(t: i32) -> AfError {
@@ -20,7 +39,7 @@ impl From<i32> for DType {
 
 impl From<i32> for InterpType {
     fn from(t: i32) -> InterpType {
-        assert!(InterpType::NEAREST as i32 <= t && t <= InterpType::CUBIC as i32);
+        assert!(InterpType::NEAREST as i32 <= t && t <= InterpType::BICUBIC_SPLINE as i32);
         unsafe { mem::transmute(t) }
     }
 }
@@ -117,3 +136,24 @@ impl_has_af_enum!(i64, DType::S64);
 impl_has_af_enum!(u64, DType::U64);
 impl_has_af_enum!(i16, DType::S16);
 impl_has_af_enum!(u16, DType::U16);
+
+impl From<i32> for SparseFormat {
+    fn from(t: i32) -> SparseFormat {
+        assert!(SparseFormat::DENSE as i32 <= t && t <= SparseFormat::COO as i32);
+        unsafe { mem::transmute(t) }
+    }
+}
+
+impl From<i32> for BinaryOp {
+    fn from(t: i32) -> BinaryOp {
+        assert!(BinaryOp::ADD as i32 <= t && t <= BinaryOp::MAX as i32);
+        unsafe { mem::transmute(t) }
+    }
+}
+
+impl From<i32> for RandomEngineType {
+    fn from(t: i32) -> RandomEngineType {
+        assert!(RandomEngineType::PHILOX_4X32_10 as i32 <= t && t <= RandomEngineType::MERSENNE_GP11213 as i32);
+        unsafe { mem::transmute(t) }
+    }
+}
